@@ -15,14 +15,16 @@ from paramiko import SSHConfig
 import pytest  # for mark
 from pytest import skip, param
 from pytest_relaxed import raises
+from invoke.vendor.lexicon import Lexicon
 
 from invoke.config import Config as InvokeConfig
 from invoke.exceptions import ThreadException
 
 from fabric import Config as Config_
+from fabric.exceptions import InvalidV1Env
 from fabric.util import get_local_user
 
-from _util import support, Connection, Config
+from _util import support, Connection, Config, faux_v1_env
 
 
 # Remote is woven in as a config default, so must be patched there
@@ -419,6 +421,39 @@ class Connection_:
                     assert middle == Connection("jumpuser2@jumphost2:872")
                     assert outermost == Connection("jumpuser@jumphost:373")
 
+                def wildcards_do_not_trigger_recursion(self):
+                    # When #1850 is present, this will RecursionError.
+                    conf = self._runtime_config(basename="proxyjump_recursive")
+                    cxn = Connection("runtime.tld", config=conf)
+                    assert cxn.gateway == Connection("bastion.tld")
+                    assert cxn.gateway.gateway is None
+
+                def multihop_plus_wildcards_still_no_recursion(self):
+                    conf = self._runtime_config(
+                        basename="proxyjump_multi_recursive"
+                    )
+                    cxn = Connection("runtime.tld", config=conf)
+                    outer = cxn.gateway
+                    inner = cxn.gateway.gateway
+                    assert outer == Connection("bastion1.tld")
+                    assert inner == Connection("bastion2.tld")
+                    assert inner.gateway is None
+
+                def gateway_Connections_get_parent_connection_configs(self):
+                    conf = self._runtime_config(
+                        basename="proxyjump",
+                        overrides={"some_random_option": "a-value"},
+                    )
+                    cxn = Connection("runtime", config=conf)
+                    # Sanity
+                    assert cxn.config is conf
+                    assert cxn.gateway == self._expected_gw
+                    # Real check
+                    assert cxn.gateway.config.some_random_option == "a-value"
+                    # Prove copy not reference
+                    # TODO: would we ever WANT a reference? can't imagine...
+                    assert cxn.gateway.config is not conf
+
             class connect_timeout:
                 def wins_over_default(self):
                     assert self._runtime_cxn().connect_timeout == 15
@@ -478,6 +513,79 @@ class Connection_:
                 cxn = Connection("host", inline_ssh_env=True)
                 assert cxn.inline_ssh_env is True
 
+    class from_v1:
+        def setup(self):
+            self.env = faux_v1_env()
+
+        def _cxn(self, **kwargs):
+            self.env.update(kwargs)
+            return Connection.from_v1(self.env)
+
+        def must_be_given_explicit_env_arg(self):
+            cxn = Connection.from_v1(self.env)
+            assert cxn.host == "localghost"
+
+        class obtaining_config:
+            @patch("fabric.connection.Config.from_v1")
+            def defaults_to_calling_Config_from_v1(self, Config_from_v1):
+                Connection.from_v1(self.env)
+                Config_from_v1.assert_called_once_with(self.env)
+
+            @patch("fabric.connection.Config.from_v1")
+            def may_be_given_config_explicitly(self, Config_from_v1):
+                # Arguably a dupe of regular Connection constructor behavior,
+                # but whatever.
+                Connection.from_v1(env=self.env, config=Config())
+                assert not Config_from_v1.called
+
+        class additional_kwargs:
+            # I.e. as opposed to what happens to the 'env' kwarg...
+            def forwards_arbitrary_kwargs_to_init(self):
+                cxn = Connection.from_v1(
+                    self.env,
+                    connect_kwargs={"foo": "bar"},
+                    inline_ssh_env=True,
+                    connect_timeout=15,
+                )
+                assert cxn.connect_kwargs["foo"] == "bar"
+                assert cxn.inline_ssh_env is True
+                assert cxn.connect_timeout == 15
+
+            def conflicting_kwargs_win_over_v1_env_values(self):
+                env = Lexicon(self.env)
+                cxn = Connection.from_v1(
+                    env, host="not-localghost", port=2222, user="remoteuser"
+                )
+                assert cxn.host == "not-localghost"
+                assert cxn.user == "remoteuser"
+                assert cxn.port == 2222
+
+        class var_mappings:
+            def host_string(self):
+                cxn = self._cxn()  # default is 'localghost'
+                assert cxn.host == "localghost"
+
+            @raises(InvalidV1Env)
+            def None_host_string_errors_usefully(self):
+                self._cxn(host_string=None)
+
+            def user(self):
+                cxn = self._cxn(user="space")
+                assert cxn.user == "space"
+
+            class port:
+                def basic(self):
+                    cxn = self._cxn(port=2222)
+                    assert cxn.port == 2222
+
+                def casted_to_int(self):
+                    cxn = self._cxn(port="2222")
+                    assert cxn.port == 2222
+
+                def not_supplied_if_given_in_host_string(self):
+                    cxn = self._cxn(host_string="localghost:3737", port=2222)
+                    assert cxn.port == 3737
+
     class string_representation:
         "string representations"
 
